/****************************************************************************
** Meta object code from reading C++ file 'ServerCatchcopy.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "catchcopy-api-0002/ServerCatchcopy.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ServerCatchcopy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ServerCatchcopy_t {
    QByteArrayData data[48];
    char stringdata0[645];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ServerCatchcopy_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ServerCatchcopy_t qt_meta_stringdata_ServerCatchcopy = {
    {
QT_MOC_LITERAL(0, 0, 15), // "ServerCatchcopy"
QT_MOC_LITERAL(1, 16, 15), // "connectedClient"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 8), // "uint32_t"
QT_MOC_LITERAL(4, 42, 2), // "id"
QT_MOC_LITERAL(5, 45, 18), // "disconnectedClient"
QT_MOC_LITERAL(6, 64, 8), // "newQuery"
QT_MOC_LITERAL(7, 73, 6), // "client"
QT_MOC_LITERAL(8, 80, 7), // "orderId"
QT_MOC_LITERAL(9, 88, 24), // "std::vector<std::string>"
QT_MOC_LITERAL(10, 113, 10), // "returnList"
QT_MOC_LITERAL(11, 124, 5), // "error"
QT_MOC_LITERAL(12, 130, 11), // "std::string"
QT_MOC_LITERAL(13, 142, 18), // "communicationError"
QT_MOC_LITERAL(14, 161, 24), // "askProtocolCompatibility"
QT_MOC_LITERAL(15, 186, 7), // "version"
QT_MOC_LITERAL(16, 194, 20), // "askProtocolExtension"
QT_MOC_LITERAL(17, 215, 9), // "extension"
QT_MOC_LITERAL(18, 225, 10), // "clientName"
QT_MOC_LITERAL(19, 236, 4), // "name"
QT_MOC_LITERAL(20, 241, 13), // "askServerName"
QT_MOC_LITERAL(21, 255, 25), // "newCopyWithoutDestination"
QT_MOC_LITERAL(22, 281, 13), // "globalOrderId"
QT_MOC_LITERAL(23, 295, 7), // "sources"
QT_MOC_LITERAL(24, 303, 7), // "newCopy"
QT_MOC_LITERAL(25, 311, 11), // "destination"
QT_MOC_LITERAL(26, 323, 25), // "newMoveWithoutDestination"
QT_MOC_LITERAL(27, 349, 7), // "newMove"
QT_MOC_LITERAL(28, 357, 13), // "newConnection"
QT_MOC_LITERAL(29, 371, 15), // "connectionError"
QT_MOC_LITERAL(30, 387, 30), // "QLocalSocket::LocalSocketError"
QT_MOC_LITERAL(31, 418, 12), // "disconnected"
QT_MOC_LITERAL(32, 431, 9), // "readyRead"
QT_MOC_LITERAL(33, 441, 12), // "checkTimeOut"
QT_MOC_LITERAL(34, 454, 16), // "disconnectClient"
QT_MOC_LITERAL(35, 471, 5), // "reply"
QT_MOC_LITERAL(36, 477, 10), // "returnCode"
QT_MOC_LITERAL(37, 488, 12), // "returnString"
QT_MOC_LITERAL(38, 501, 17), // "protocolSupported"
QT_MOC_LITERAL(39, 519, 5), // "value"
QT_MOC_LITERAL(40, 525, 25), // "incorrectArgumentListSize"
QT_MOC_LITERAL(41, 551, 17), // "incorrectArgument"
QT_MOC_LITERAL(42, 569, 16), // "clientRegistered"
QT_MOC_LITERAL(43, 586, 10), // "serverName"
QT_MOC_LITERAL(44, 597, 12), // "copyFinished"
QT_MOC_LITERAL(45, 610, 9), // "withError"
QT_MOC_LITERAL(46, 620, 12), // "copyCanceled"
QT_MOC_LITERAL(47, 633, 11) // "unknowOrder"

    },
    "ServerCatchcopy\0connectedClient\0\0"
    "uint32_t\0id\0disconnectedClient\0newQuery\0"
    "client\0orderId\0std::vector<std::string>\0"
    "returnList\0error\0std::string\0"
    "communicationError\0askProtocolCompatibility\0"
    "version\0askProtocolExtension\0extension\0"
    "clientName\0name\0askServerName\0"
    "newCopyWithoutDestination\0globalOrderId\0"
    "sources\0newCopy\0destination\0"
    "newMoveWithoutDestination\0newMove\0"
    "newConnection\0connectionError\0"
    "QLocalSocket::LocalSocketError\0"
    "disconnected\0readyRead\0checkTimeOut\0"
    "disconnectClient\0reply\0returnCode\0"
    "returnString\0protocolSupported\0value\0"
    "incorrectArgumentListSize\0incorrectArgument\0"
    "clientRegistered\0serverName\0copyFinished\0"
    "withError\0copyCanceled\0unknowOrder"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ServerCatchcopy[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      32,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      14,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  174,    2, 0x06 /* Public */,
       5,    1,  177,    2, 0x06 /* Public */,
       6,    3,  180,    2, 0x06 /* Public */,
      11,    1,  187,    2, 0x06 /* Public */,
      13,    1,  190,    2, 0x06 /* Public */,
      14,    3,  193,    2, 0x06 /* Public */,
      16,    3,  200,    2, 0x06 /* Public */,
      16,    4,  207,    2, 0x06 /* Public */,
      18,    2,  216,    2, 0x06 /* Public */,
      20,    2,  221,    2, 0x06 /* Public */,
      21,    2,  226,    2, 0x06 /* Public */,
      24,    3,  231,    2, 0x06 /* Public */,
      26,    2,  238,    2, 0x06 /* Public */,
      27,    3,  243,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      28,    0,  250,    2, 0x08 /* Private */,
      29,    1,  251,    2, 0x08 /* Private */,
      31,    0,  254,    2, 0x08 /* Private */,
      32,    0,  255,    2, 0x08 /* Private */,
      33,    0,  256,    2, 0x08 /* Private */,
      34,    1,  257,    2, 0x0a /* Public */,
      35,    4,  260,    2, 0x0a /* Public */,
      35,    4,  269,    2, 0x0a /* Public */,
      38,    3,  278,    2, 0x0a /* Public */,
      40,    2,  285,    2, 0x0a /* Public */,
      41,    2,  290,    2, 0x0a /* Public */,
      42,    2,  295,    2, 0x0a /* Public */,
      43,    3,  300,    2, 0x0a /* Public */,
      44,    3,  307,    2, 0x0a /* Public */,
      46,    2,  314,    2, 0x0a /* Public */,
      44,    2,  319,    2, 0x0a /* Public */,
      46,    1,  324,    2, 0x0a /* Public */,
      47,    2,  327,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 9,    7,    8,   10,
    QMetaType::Void, 0x80000000 | 12,   11,
    QMetaType::Void, 0x80000000 | 12,   11,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 12,    7,    8,   15,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 12,    7,    8,   17,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 12, 0x80000000 | 12,    7,    8,   17,   15,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 12,    7,   19,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    7,    8,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 9,   22,   23,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 9, 0x80000000 | 12,   22,   23,   25,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 9,   22,   23,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 9, 0x80000000 | 12,   22,   23,   25,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 30,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 9,    7,    8,   36,   10,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 12,    7,    8,   36,   37,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, QMetaType::Bool,    7,    8,   39,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    7,    8,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    7,    8,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    7,    8,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, 0x80000000 | 12,    7,    8,   19,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3, QMetaType::Bool,    7,    8,   45,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    7,    8,
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,   22,   45,
    QMetaType::Void, 0x80000000 | 3,   22,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    7,    8,

       0        // eod
};

void ServerCatchcopy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ServerCatchcopy *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->connectedClient((*reinterpret_cast< const uint32_t(*)>(_a[1]))); break;
        case 1: _t->disconnectedClient((*reinterpret_cast< const uint32_t(*)>(_a[1]))); break;
        case 2: _t->newQuery((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[3]))); break;
        case 3: _t->error((*reinterpret_cast< const std::string(*)>(_a[1]))); break;
        case 4: _t->communicationError((*reinterpret_cast< const std::string(*)>(_a[1]))); break;
        case 5: _t->askProtocolCompatibility((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3]))); break;
        case 6: _t->askProtocolExtension((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3]))); break;
        case 7: _t->askProtocolExtension((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3])),(*reinterpret_cast< const std::string(*)>(_a[4]))); break;
        case 8: _t->clientName((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::string(*)>(_a[2]))); break;
        case 9: _t->askServerName((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2]))); break;
        case 10: _t->newCopyWithoutDestination((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2]))); break;
        case 11: _t->newCopy((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3]))); break;
        case 12: _t->newMoveWithoutDestination((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2]))); break;
        case 13: _t->newMove((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3]))); break;
        case 14: _t->newConnection(); break;
        case 15: _t->connectionError((*reinterpret_cast< const QLocalSocket::LocalSocketError(*)>(_a[1]))); break;
        case 16: _t->disconnected(); break;
        case 17: _t->readyRead(); break;
        case 18: _t->checkTimeOut(); break;
        case 19: _t->disconnectClient((*reinterpret_cast< const uint32_t(*)>(_a[1]))); break;
        case 20: _t->reply((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2])),(*reinterpret_cast< const uint32_t(*)>(_a[3])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[4]))); break;
        case 21: _t->reply((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2])),(*reinterpret_cast< const uint32_t(*)>(_a[3])),(*reinterpret_cast< const std::string(*)>(_a[4]))); break;
        case 22: _t->protocolSupported((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2])),(*reinterpret_cast< const bool(*)>(_a[3]))); break;
        case 23: _t->incorrectArgumentListSize((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2]))); break;
        case 24: _t->incorrectArgument((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2]))); break;
        case 25: _t->clientRegistered((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2]))); break;
        case 26: _t->serverName((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3]))); break;
        case 27: _t->copyFinished((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2])),(*reinterpret_cast< const bool(*)>(_a[3]))); break;
        case 28: _t->copyCanceled((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2]))); break;
        case 29: _t->copyFinished((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 30: _t->copyCanceled((*reinterpret_cast< const uint32_t(*)>(_a[1]))); break;
        case 31: _t->unknowOrder((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const uint32_t(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::connectedClient)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::disconnectedClient)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & , const uint32_t & , const std::vector<std::string> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::newQuery)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const std::string & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::error)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const std::string & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::communicationError)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & , const uint32_t & , const std::string & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::askProtocolCompatibility)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & , const uint32_t & , const std::string & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::askProtocolExtension)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & , const uint32_t & , const std::string & , const std::string & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::askProtocolExtension)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & , const std::string & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::clientName)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & , const uint32_t & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::askServerName)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & , const std::vector<std::string> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::newCopyWithoutDestination)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & , const std::vector<std::string> & , const std::string & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::newCopy)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & , const std::vector<std::string> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::newMoveWithoutDestination)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (ServerCatchcopy::*)(const uint32_t & , const std::vector<std::string> & , const std::string & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ServerCatchcopy::newMove)) {
                *result = 13;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ServerCatchcopy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ServerCatchcopy.data,
    qt_meta_data_ServerCatchcopy,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ServerCatchcopy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ServerCatchcopy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ServerCatchcopy.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ServerCatchcopy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 32)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 32;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 32)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 32;
    }
    return _id;
}

// SIGNAL 0
void ServerCatchcopy::connectedClient(const uint32_t & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ServerCatchcopy::disconnectedClient(const uint32_t & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ServerCatchcopy::newQuery(const uint32_t & _t1, const uint32_t & _t2, const std::vector<std::string> & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ServerCatchcopy::error(const std::string & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ServerCatchcopy::communicationError(const std::string & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void ServerCatchcopy::askProtocolCompatibility(const uint32_t & _t1, const uint32_t & _t2, const std::string & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void ServerCatchcopy::askProtocolExtension(const uint32_t & _t1, const uint32_t & _t2, const std::string & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void ServerCatchcopy::askProtocolExtension(const uint32_t & _t1, const uint32_t & _t2, const std::string & _t3, const std::string & _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void ServerCatchcopy::clientName(const uint32_t & _t1, const std::string & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void ServerCatchcopy::askServerName(const uint32_t & _t1, const uint32_t & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void ServerCatchcopy::newCopyWithoutDestination(const uint32_t & _t1, const std::vector<std::string> & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void ServerCatchcopy::newCopy(const uint32_t & _t1, const std::vector<std::string> & _t2, const std::string & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void ServerCatchcopy::newMoveWithoutDestination(const uint32_t & _t1, const std::vector<std::string> & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void ServerCatchcopy::newMove(const uint32_t & _t1, const std::vector<std::string> & _t2, const std::string & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
