//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by CatchCopy.rc
//
#define IDS_PROJNAME                    100
#define IDR_CATCHCOPY                 101
#define IDR_DDSHELLEXT                  102

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        201
#define _APS_NEXT_COMMAND_VALUE         32768
#define _APS_NEXT_CONTROL_VALUE         201
#define _APS_NEXT_SYMED_VALUE           103
#endif
#endif
