/****************************************************************************
** Meta object code from reading C++ file 'listener.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "listener.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'listener.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Listener_t {
    QByteArrayData data[22];
    char stringdata0[273];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Listener_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Listener_t qt_meta_stringdata_Listener = {
    {
QT_MOC_LITERAL(0, 0, 8), // "Listener"
QT_MOC_LITERAL(1, 9, 16), // "transferFinished"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 8), // "uint32_t"
QT_MOC_LITERAL(4, 36, 7), // "orderId"
QT_MOC_LITERAL(5, 44, 9), // "withError"
QT_MOC_LITERAL(6, 54, 16), // "transferCanceled"
QT_MOC_LITERAL(7, 71, 17), // "newLanguageLoaded"
QT_MOC_LITERAL(8, 89, 13), // "errorInternal"
QT_MOC_LITERAL(9, 103, 11), // "std::string"
QT_MOC_LITERAL(10, 115, 6), // "string"
QT_MOC_LITERAL(11, 122, 26), // "communicationErrorInternal"
QT_MOC_LITERAL(12, 149, 10), // "clientName"
QT_MOC_LITERAL(13, 160, 6), // "client"
QT_MOC_LITERAL(14, 167, 4), // "name"
QT_MOC_LITERAL(15, 172, 22), // "copyWithoutDestination"
QT_MOC_LITERAL(16, 195, 24), // "std::vector<std::string>"
QT_MOC_LITERAL(17, 220, 7), // "sources"
QT_MOC_LITERAL(18, 228, 4), // "copy"
QT_MOC_LITERAL(19, 233, 11), // "destination"
QT_MOC_LITERAL(20, 245, 22), // "moveWithoutDestination"
QT_MOC_LITERAL(21, 268, 4) // "move"

    },
    "Listener\0transferFinished\0\0uint32_t\0"
    "orderId\0withError\0transferCanceled\0"
    "newLanguageLoaded\0errorInternal\0"
    "std::string\0string\0communicationErrorInternal\0"
    "clientName\0client\0name\0copyWithoutDestination\0"
    "std::vector<std::string>\0sources\0copy\0"
    "destination\0moveWithoutDestination\0"
    "move"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Listener[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   64,    2, 0x0a /* Public */,
       6,    1,   69,    2, 0x0a /* Public */,
       7,    0,   72,    2, 0x0a /* Public */,
       8,    1,   73,    2, 0x08 /* Private */,
      11,    1,   76,    2, 0x08 /* Private */,
      12,    2,   79,    2, 0x08 /* Private */,
      15,    2,   84,    2, 0x08 /* Private */,
      18,    3,   89,    2, 0x08 /* Private */,
      20,    2,   96,    2, 0x08 /* Private */,
      21,    3,  101,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    4,    5,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 9,   13,   14,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 16,    4,   17,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 16, 0x80000000 | 9,    4,   17,   19,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 16,    4,   17,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 16, 0x80000000 | 9,    4,   17,   19,

       0        // eod
};

void Listener::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Listener *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->transferFinished((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 1: _t->transferCanceled((*reinterpret_cast< const uint32_t(*)>(_a[1]))); break;
        case 2: _t->newLanguageLoaded(); break;
        case 3: _t->errorInternal((*reinterpret_cast< const std::string(*)>(_a[1]))); break;
        case 4: _t->communicationErrorInternal((*reinterpret_cast< const std::string(*)>(_a[1]))); break;
        case 5: _t->clientName((*reinterpret_cast< uint32_t(*)>(_a[1])),(*reinterpret_cast< std::string(*)>(_a[2]))); break;
        case 6: _t->copyWithoutDestination((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2]))); break;
        case 7: _t->copy((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3]))); break;
        case 8: _t->moveWithoutDestination((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2]))); break;
        case 9: _t->move((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Listener::staticMetaObject = { {
    QMetaObject::SuperData::link<PluginInterface_Listener::staticMetaObject>(),
    qt_meta_stringdata_Listener.data,
    qt_meta_data_Listener,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Listener::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Listener::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Listener.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "first-world.info.ultracopier.PluginInterface.Listener/2.0.0.0"))
        return static_cast< PluginInterface_Listener*>(this);
    return PluginInterface_Listener::qt_metacast(_clname);
}

int Listener::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PluginInterface_Listener::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

QT_PLUGIN_METADATA_SECTION
static constexpr unsigned char qt_pluginMetaData[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x3d,  'f',  'i',  'r',  's',  't', 
    '-',  'w',  'o',  'r',  'l',  'd',  '.',  'i', 
    'n',  'f',  'o',  '.',  'u',  'l',  't',  'r', 
    'a',  'c',  'o',  'p',  'i',  'e',  'r',  '.', 
    'P',  'l',  'u',  'g',  'i',  'n',  'I',  'n', 
    't',  'e',  'r',  'f',  'a',  'c',  'e',  '.', 
    'L',  'i',  's',  't',  'e',  'n',  'e',  'r', 
    '/',  '1',  '.',  '0',  '.',  '0',  '.',  '0', 
    // "className"
    0x03,  0x68,  'L',  'i',  's',  't',  'e',  'n', 
    'e',  'r', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(Listener, Listener)

QT_WARNING_POP
QT_END_MOC_NAMESPACE
