/****************************************************************************
** Meta object code from reading C++ file 'PluginInterface_Listener.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../interface/PluginInterface_Listener.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PluginInterface_Listener.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PluginInterface_Listener_t {
    QByteArrayData data[28];
    char stringdata0[365];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PluginInterface_Listener_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PluginInterface_Listener_t qt_meta_stringdata_PluginInterface_Listener = {
    {
QT_MOC_LITERAL(0, 0, 24), // "PluginInterface_Listener"
QT_MOC_LITERAL(1, 25, 8), // "newState"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 27), // "Ultracopier::ListeningState"
QT_MOC_LITERAL(4, 63, 5), // "state"
QT_MOC_LITERAL(5, 69, 25), // "newCopyWithoutDestination"
QT_MOC_LITERAL(6, 95, 8), // "uint32_t"
QT_MOC_LITERAL(7, 104, 7), // "orderId"
QT_MOC_LITERAL(8, 112, 24), // "std::vector<std::string>"
QT_MOC_LITERAL(9, 137, 7), // "sources"
QT_MOC_LITERAL(10, 145, 7), // "newCopy"
QT_MOC_LITERAL(11, 153, 11), // "std::string"
QT_MOC_LITERAL(12, 165, 11), // "destination"
QT_MOC_LITERAL(13, 177, 25), // "newMoveWithoutDestination"
QT_MOC_LITERAL(14, 203, 7), // "newMove"
QT_MOC_LITERAL(15, 211, 5), // "error"
QT_MOC_LITERAL(16, 217, 13), // "newClientList"
QT_MOC_LITERAL(17, 231, 16), // "debugInformation"
QT_MOC_LITERAL(18, 248, 23), // "Ultracopier::DebugLevel"
QT_MOC_LITERAL(19, 272, 5), // "level"
QT_MOC_LITERAL(20, 278, 8), // "fonction"
QT_MOC_LITERAL(21, 287, 4), // "text"
QT_MOC_LITERAL(22, 292, 4), // "file"
QT_MOC_LITERAL(23, 297, 5), // "ligne"
QT_MOC_LITERAL(24, 303, 16), // "transferFinished"
QT_MOC_LITERAL(25, 320, 9), // "withError"
QT_MOC_LITERAL(26, 330, 16), // "transferCanceled"
QT_MOC_LITERAL(27, 347, 17) // "newLanguageLoaded"

    },
    "PluginInterface_Listener\0newState\0\0"
    "Ultracopier::ListeningState\0state\0"
    "newCopyWithoutDestination\0uint32_t\0"
    "orderId\0std::vector<std::string>\0"
    "sources\0newCopy\0std::string\0destination\0"
    "newMoveWithoutDestination\0newMove\0"
    "error\0newClientList\0debugInformation\0"
    "Ultracopier::DebugLevel\0level\0fonction\0"
    "text\0file\0ligne\0transferFinished\0"
    "withError\0transferCanceled\0newLanguageLoaded"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PluginInterface_Listener[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,
       5,    2,   72,    2, 0x06 /* Public */,
      10,    3,   77,    2, 0x06 /* Public */,
      13,    2,   84,    2, 0x06 /* Public */,
      14,    3,   89,    2, 0x06 /* Public */,
      15,    1,   96,    2, 0x06 /* Public */,
      16,    0,   99,    2, 0x06 /* Public */,
      17,    5,  100,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      24,    2,  111,    2, 0x0a /* Public */,
      26,    1,  116,    2, 0x0a /* Public */,
      27,    0,  119,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 8,    7,    9,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 8, 0x80000000 | 11,    7,    9,   12,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 8,    7,    9,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 8, 0x80000000 | 11,    7,    9,   12,
    QMetaType::Void, 0x80000000 | 11,   15,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18, 0x80000000 | 11, 0x80000000 | 11, 0x80000000 | 11, QMetaType::Int,   19,   20,   21,   22,   23,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 6, QMetaType::Bool,    7,   25,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,

       0        // eod
};

void PluginInterface_Listener::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PluginInterface_Listener *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->newState((*reinterpret_cast< const Ultracopier::ListeningState(*)>(_a[1]))); break;
        case 1: _t->newCopyWithoutDestination((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2]))); break;
        case 2: _t->newCopy((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3]))); break;
        case 3: _t->newMoveWithoutDestination((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2]))); break;
        case 4: _t->newMove((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const std::vector<std::string>(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3]))); break;
        case 5: _t->error((*reinterpret_cast< const std::string(*)>(_a[1]))); break;
        case 6: _t->newClientList(); break;
        case 7: _t->debugInformation((*reinterpret_cast< const Ultracopier::DebugLevel(*)>(_a[1])),(*reinterpret_cast< const std::string(*)>(_a[2])),(*reinterpret_cast< const std::string(*)>(_a[3])),(*reinterpret_cast< const std::string(*)>(_a[4])),(*reinterpret_cast< const int(*)>(_a[5]))); break;
        case 8: _t->transferFinished((*reinterpret_cast< const uint32_t(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2]))); break;
        case 9: _t->transferCanceled((*reinterpret_cast< const uint32_t(*)>(_a[1]))); break;
        case 10: _t->newLanguageLoaded(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PluginInterface_Listener::*)(const Ultracopier::ListeningState & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PluginInterface_Listener::newState)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PluginInterface_Listener::*)(const uint32_t & , const std::vector<std::string> & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PluginInterface_Listener::newCopyWithoutDestination)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PluginInterface_Listener::*)(const uint32_t & , const std::vector<std::string> & , const std::string & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PluginInterface_Listener::newCopy)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (PluginInterface_Listener::*)(const uint32_t & , const std::vector<std::string> & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PluginInterface_Listener::newMoveWithoutDestination)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (PluginInterface_Listener::*)(const uint32_t & , const std::vector<std::string> & , const std::string & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PluginInterface_Listener::newMove)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (PluginInterface_Listener::*)(const std::string & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PluginInterface_Listener::error)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (PluginInterface_Listener::*)() const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PluginInterface_Listener::newClientList)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (PluginInterface_Listener::*)(const Ultracopier::DebugLevel & , const std::string & , const std::string & , const std::string & , const int & ) const;
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PluginInterface_Listener::debugInformation)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PluginInterface_Listener::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_PluginInterface_Listener.data,
    qt_meta_data_PluginInterface_Listener,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PluginInterface_Listener::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PluginInterface_Listener::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PluginInterface_Listener.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PluginInterface_Listener::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void PluginInterface_Listener::newState(const Ultracopier::ListeningState & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< PluginInterface_Listener *>(this), &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PluginInterface_Listener::newCopyWithoutDestination(const uint32_t & _t1, const std::vector<std::string> & _t2)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(const_cast< PluginInterface_Listener *>(this), &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PluginInterface_Listener::newCopy(const uint32_t & _t1, const std::vector<std::string> & _t2, const std::string & _t3)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(const_cast< PluginInterface_Listener *>(this), &staticMetaObject, 2, _a);
}

// SIGNAL 3
void PluginInterface_Listener::newMoveWithoutDestination(const uint32_t & _t1, const std::vector<std::string> & _t2)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(const_cast< PluginInterface_Listener *>(this), &staticMetaObject, 3, _a);
}

// SIGNAL 4
void PluginInterface_Listener::newMove(const uint32_t & _t1, const std::vector<std::string> & _t2, const std::string & _t3)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(const_cast< PluginInterface_Listener *>(this), &staticMetaObject, 4, _a);
}

// SIGNAL 5
void PluginInterface_Listener::error(const std::string & _t1)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(const_cast< PluginInterface_Listener *>(this), &staticMetaObject, 5, _a);
}

// SIGNAL 6
void PluginInterface_Listener::newClientList()const
{
    QMetaObject::activate(const_cast< PluginInterface_Listener *>(this), &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void PluginInterface_Listener::debugInformation(const Ultracopier::DebugLevel & _t1, const std::string & _t2, const std::string & _t3, const std::string & _t4, const int & _t5)const
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))) };
    QMetaObject::activate(const_cast< PluginInterface_Listener *>(this), &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
