/** \file VariablesCatchcopy.h
\brief Define the variable for catchcopy
\author alpha_one_x86
\licence GPL3, see the file COPYING */

#ifndef VARIABLECATCHCOPY_H
#define VARIABLECATCHCOPY_H

#define CATCHCOPY_PROTOCOL_VERSION "0002"
#define CATCHCOPY_COMMUNICATION_TIMEOUT 200

#endif // VARIABLECATCHCOPY_H

