#ifndef VERSIONUC_H
#define VERSIONUC_H

/// \brief the version
#define ULTRACOPIER_VERSION		"2.2.6.0"
/// \brief the windows version
#define ULTRACOPIER_WINDOWS_VERSION	2,2,6,0

#endif // VARIABLE_H
