#include <windows.h>
#include "../Version.h"

1 VERSIONINFO
FILEVERSION ULTRACOPIER_WINDOWS_VERSION
PRODUCTVERSION ULTRACOPIER_WINDOWS_VERSION
FILEFLAGSMASK 0x3fL
FILETYPE VFT_APP
{
	BLOCK "StringFileInfo"
	{
		BLOCK "040904E4"
		{
			VALUE "CompanyName", "ultracopier.first-world.info"
			VALUE "FileVersion", ULTRACOPIER_VERSION
			VALUE "FileDescription", "Ultracopier under GPL3"
			VALUE "InternalName", "ultracopier.exe"
			VALUE "LegalCopyright", "GPL3"
			VALUE "LegalTrademarks", "Ultracopier under GPL3"
			VALUE "OriginalFilename", "ultracopier.exe"
			VALUE "ProductName", "Ultracopier"
			VALUE "ProductVersion", ULTRACOPIER_VERSION
		}
	}
	BLOCK "VarFileInfo"
	{
		VALUE "Translation", 0x0409, 1252
	}
}

#ifdef ULTRACOPIER_PLUGIN_ALL_IN_ONE
IDI_ICON1               ICON    DISCARDABLE     "ultracopier-all-in-one.ico"
#else
IDI_ICON1               ICON    DISCARDABLE     "ultracopier.ico"
#endif

