/** \file Environment.h
\brief Define the environment variable and global function
\author alpha_one_x86
\licence GPL3, see the file COPYING */

#include "Oxygen2Variable.h"
/// \brief The global include
#include "StructEnumDefinition.h"
#include "DebugEngineMacro.h"

